/*
 * Copyright (c) 2003-2003
 *	Fraunhofer Institute for Open Communication Systems (FhG Fokus).
 * 	All rights reserved.
 *
 * Author: Hartmut Brandt <harti@freebsd.org>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 * $Begemot: atmsupport/libunitcl/api.c,v 1.2 2004/01/02 15:15:54 hbb Exp $
 */
#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <string.h>
#include <unistd.h>

#include "inctcl.h"

#include <netnatm/uni/unimsg.h>
#include <netnatm/uni/unistruct.h>
#include <netnatm/uni/unimsglib.h>
#include <netnatm/uni/unisap.h>
#include <netnatm/uni/unidef.h>
#include <netnatm/uni/atmapi.h>

#include "unitcl.h"
#include "unitcl_priv.h"

#if 0

/* maximum number of attributes to set/get at once */
#define ATM_MAX_ATTR	40

enum atmstate {
	ATM_A0,		/* non existent */
	ATM_A1,		/* initial */
	ATM_A2,		/* outgoing call preparation */
	ATM_A3,		/* outgoing call requested */
	ATM_A4,		/* incoming call preparation */
	ATM_A5,		/* wait incoming call */
	ATM_A6,		/* incoming call present */
	ATM_A7,		/* incoming call requested */
	ATM_A8,		/* p2p data transfer */
	ATM_A9,		/* p2mp root data transfer */
	ATM_A10,	/* p2mp leaf data transfer */
	ATM_A11,	/* terminated */
};

enum atmop {
	ATMOP_RESP,				/* 0 */
	ATMOP_ABORT_CONNECTION,
	ATMOP_ACCEPT_INCOMING_CALL,
	ATMOP_ADD_PARTY,
	ATMOP_ADD_PARTY_REJECT,
	ATMOP_ADD_PARTY_SUCCESS,		/* 5 */
	ATMOP_ARRIVAL_OF_INCOMING_CALL,
	ATMOP_CALL_RELEASE,
	ATMOP_CONNECT_OUTGOING_CALL,
	ATMOP_DROP_PARTY,
	ATMOP_GET_LOCAL_PORT_INFO,		/* 10 */
	ATMOP_P2MP_CALL_ACTIVE,
	ATMOP_P2P_CALL_ACTIVE,
	ATMOP_PREPARE_INCOMING_CALL,
	ATMOP_PREPARE_OUTGOING_CALL,
	ATMOP_QUERY_CONNECTION_ATTRIBUTES,	/* 15 */
	ATMOP_REJECT_INCOMING_CALL,
	ATMOP_SET_CONNECTION_ATTRIBUTES,
	ATMOP_WAIT_ON_INCOMING_CALL,
	ATMOP_SET_CONNECTION_ATTRIBUTES_X,
	ATMOP_QUERY_CONNECTION_ATTRIBUTES_X,	/* 20 */
};

#define ATM_DEFINE_ERRORS						\
   DEF(ATMERR_OK,		 0, "OK")				\
   DEF(ATMERR_SYS,		-1, "syscall error")			\
   DEF(ATMERR_BAD_OP,		-2, "bad operation")			\
   DEF(ATMERR_BAD_ARGS,		-3, "bad arguments for operation")	\
   DEF(ATMERR_BAD_STATE,	-4, "operation in bad state")		\
   DEF(ATMERR_BAD_ATTR,		-5, "unknown attribute")		\
   DEF(ATMERR_BAD_VALUE,	-6, "bad attribute value")		\
   DEF(ATMERR_BUSY,		-7, "busy")				\
   DEF(ATMERR_RDONLY,		-8, "read-only attribute")		\
   DEF(ATMERR_BAD_SAP,		-9, "bad SAP")				\
   DEF(ATMERR_OVERLAP,		-10,"overlaping SAP")			\
   DEF(ATMERR_BAD_ENDPOINT,	-11,"bad ATM endpoint")			\
   DEF(ATMERR_PREVIOUSLY_ABORTED,-12,"previously aborted")		\
   DEF(ATMERR_NO_CALL,		-13,"no incoming call")			\
   DEF(ATMERR_BAD_LEAF_IDENT,	-14,"bad leaf identifier")		\
   DEF(ATMERR_BAD_PORT,		-15,"unknown port")			\
   DEF(ATMERR_BAD_SIGNAL,	-29-UNIAPI_ERROR_BAD_SIGNAL, "bad signal")\
   DEF(ATMERR_BADCU,		-29-UNIAPI_ERROR_BADCU, "bad coordinator state")\
   DEF(ATMERR_BAD_CALLSTATE,	-29-UNIAPI_ERROR_BAD_CALLSTATE, "bad call state")\
   DEF(ATMERR_BAD_EPSTATE,	-29-UNIAPI_ERROR_BAD_EPSTATE, "bad party state")\
   DEF(ATMERR_BAD_UNIARG,	-29-UNIAPI_ERROR_BAD_ARG, "bad uni argument")\
   DEF(ATMERR_BAD_CALL,		-29-UNIAPI_ERROR_BAD_CALL, "unknown call")\
   DEF(ATMERR_BAD_PARTY,	-29-UNIAPI_ERROR_BAD_PARTY, "unknown party")\
   DEF(ATMERR_BAD_CTYPE,	-29-UNIAPI_ERROR_BAD_CTYPE, "wrong call type")\
   DEF(ATMERR_BAD_IE,		-29-UNIAPI_ERROR_BAD_IE, "bad information element")\
   DEF(ATMERR_EPREF_INUSE,	-29-UNIAPI_ERROR_EPREF_INUSE, "endpoint reference in use")\
   DEF(ATMERR_MISSING_IE,	-29-UNIAPI_ERROR_MISSING_IE, "missing information element")\
   DEF(ATMERR_ENCODING,		-29-UNIAPI_ERROR_ENCODING, "encoding error")\
   DEF(ATMERR_NOMEM,		-29-UNIAPI_ERROR_NOMEM, "no memory")\
   DEF(ATMERR_UNIBUSY,		-29-UNIAPI_ERROR_BUSY, "uni process busy")

#define ATM_MKUNIERR(E)	(-29-(E))

enum atm_error {
#define DEF(NAME,VAL,STR)	NAME = (VAL),
ATM_DEFINE_ERRORS
#undef DEF
};

enum atm_attribute {
	ATM_ATTR_NONE = 0,
	ATM_ATTR_BLLI_SELECTOR,
	ATM_ATTR_BLLI,
	ATM_ATTR_BEARER,
	ATM_ATTR_TRAFFIC,
	ATM_ATTR_QOS,
	ATM_ATTR_EXQOS,
	ATM_ATTR_CALLED,
	ATM_ATTR_CALLEDSUB,
	ATM_ATTR_CALLING,
	ATM_ATTR_CALLINGSUB,
	ATM_ATTR_AAL,
	ATM_ATTR_EPREF,
	ATM_ATTR_CONNED,
	ATM_ATTR_CONNEDSUB,
	ATM_ATTR_EETD,
	ATM_ATTR_ABRSETUP,
	ATM_ATTR_ABRADD,
	ATM_ATTR_CONNID,
	ATM_ATTR_MDCR,
};

#endif

static const struct {
	int32_t	resp;
	const char *name;
	const char *txt;
} responses[] = {
#define DEF(NAME, VAL, STR)	{ NAME, #NAME, STR },
ATM_DEFINE_ERRORS
#undef DEF
};

/*
 * There are three cases of a response: get_attributes[_x], set_attributes_x
 * and get local port info.
 */
int
fmt_api_resp(Tcl_Interp *interp, Tcl_DString *str,
    const struct atm_resp *resp, size_t len, u_int op)
{
	u_int i;
	char sbuf[100];

	Tcl_DStringStartSublist(str);
	Tcl_DStringAppend(str, "api-resp", -1);

	for (i = 0; i < sizeof(responses) / sizeof(responses[0]); i++)
		if (responses[i].resp == resp->resp)
			break;

	if (i == sizeof(responses) / sizeof(responses[0])) {
		sprintf(sbuf, "%d", resp->resp);
		Tcl_DStringAppendElement(str, sbuf);
		Tcl_DStringStartSublist(str);
		Tcl_DStringAppend(str, "unknown error", -1);
		Tcl_DStringEndSublist(str);
	} else {
		Tcl_DStringAppendElement(str, responses[i].name);
		Tcl_DStringStartSublist(str);
		Tcl_DStringAppend(str, responses[i].txt, -1);
		Tcl_DStringEndSublist(str);
	}

	switch (op) {

	}
	Tcl_DStringEndSublist(str);
}

#if 0
/*
 * This structure carries the return value from the user call
 */
struct atm_resp {
	int32_t		resp;
	u_char		val[];
};

/*
 * Get/Set connection attributes
 */
struct atm_query_connection_attributes {
	uint32_t	attr;
};
struct atm_set_connection_attributes {
	uint32_t	attr;
	u_char		val[];
};
struct atm_set_connection_attributes_x {
	uint32_t	attr[ATM_MAX_ATTR];
	u_char		val[];
};
struct atm_query_connection_attributes_x {
	uint32_t	attr[ATM_MAX_ATTR];
};

struct atm_prepare_outgoing_call {
};
struct atm_prepare_incoming_call {
	struct uni_sap		sap;
	uint32_t		queue_size;
};
struct atm_connect_outgoing_call {
	struct uni_ie_called	called;
};
struct atm_call_release {
	struct uni_ie_cause	cause[2];
};
struct atm_p2p_call_active {
	struct uni_ie_connid	connid;
};
struct atm_p2mp_call_active {
	struct uni_ie_connid	connid;
};
struct atm_wait_on_incoming_call {
};
struct atm_arrival_of_incoming_call {
};
struct atm_accept_incoming_call {
	uint32_t		newep;
};
struct atm_reject_incoming_call {
	struct uni_ie_cause	cause;
};
struct atm_add_party {
	uint16_t		leaf_ident;
	struct uni_ie_called	called;
};
struct atm_add_party_success {
	uint16_t		leaf_ident;
};
struct atm_add_party_reject {
	uint16_t		leaf_ident;
	struct uni_ie_cause	cause;
};
struct atm_drop_party {
	uint16_t		leaf_ident;
	struct uni_ie_cause	cause;
};

struct atm_get_local_port_info {
	uint32_t		port;
};

struct atm_port_info {
	uint32_t		port;
	uint32_t		pcr;
	uint32_t		num_addr;
};
struct atm_port_list {
	uint32_t		num_ports;
	struct atm_port_info	ports[0];
};

#endif
